
/***********************************************************************************
*
* ItalaApi - Copyright (C) 2022 Opto Engineering
*
* THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
* INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT
* HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY SUFFERED BY LICENSE AS
* A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
*
***********************************************************************************/

#ifndef ITALA_API_SMART_H
#define ITALA_API_SMART_H

#include "ISystem.h"
#include "IDevice.h"
#include "IImage.h"

#include <memory>

/**
@file
* The following functions provide a convenient way for managing IDisposable
* instances lifecycle via the smart pointers of the C++ standard library.
* These functions should be called immediately after creating the instance to avoid
* both raw and smart pointers around at the same time.
* When the smart pointer is destroyed, the IDisposable::Dispose method of the IDisposable
* interface is automatically called.
*
* \code{.cpp}
* {
*   // here the system instance is created and passed to a smart pointer
* 	Itala::SystemPtr pSystem = Itala::SmartAssign(Itala::CreateSystem());
* } // here the system instance is automatically disposed as the pointer goes out of scope
* \endcode
*
* @warning When a IDisposable instance is owned by a smart pointer, its
* IDisposable::Dispose method must NEVER be called manually!
*
* Note that this header is just a quick helper completely editable by the user.
* Feel free to introduce improvements or entirely swap the smart pointer
* implementation with a different one (e.g. Boost).
*/

namespace Itala
{
  typedef std::shared_ptr<ISystem> SystemPtr;
  typedef std::shared_ptr<IDevice> DevicePtr;
  typedef std::shared_ptr<IImage> ImagePtr;

  inline SystemPtr SmartAssign(ISystem* pSystem)
  {
    return SystemPtr(pSystem,
      [pSystem](Itala::ISystem* sys) {
        sys->Dispose();
      });
  }

  inline DevicePtr SmartAssign(IDevice* pDevice)
  {
    return DevicePtr(pDevice,
      [pDevice](Itala::IDevice* dev) {
        dev->Dispose();
      });
  }

  inline ImagePtr SmartAssign(IImage* pImage)
  {
    return ImagePtr(pImage,
      [pImage](Itala::IImage* img) {
        img->Dispose();
      });
  }

}

#endif